////////////////////////////////////////////////////////////////////////////////
///
///		Title		: NPhoto HTTP Client
///
///		Copyright (c) 2006, NHN Co., Ltd. ALL RIGHTS RESERVED
///
///		Team		: Desktop Service Development
///////////////////////////////////////////////////////////////////////////////
#pragma once

#include	<wininet.h>
#pragma	comment(lib, "Wininet.lib")

/**
* @brief 	DownloadFile Callback function
* @param	dwBytesSent		sent data length
* @param	ulTotalLen		sent total data length
* @param	dwParam			DownloadFile's dwParam
* @return	go or stop
*/
typedef BOOL (WINAPI *PFNHTTPSTATUSCALLBACK)(DWORD dwBytesSent, ULONG ulTotalLen, DWORD_PTR dwParam);


typedef	struct _tagPostArg
{
	BSTR		bstrKey;
	BSTR		bstrValue;
	BOOL		bBinary;
}MB_POSTARG, *PMB_POSTARG;

inline BOOL NewPostArg(PMB_POSTARG* ppPostArg)
{
	PMB_POSTARG pPostArg = (PMB_POSTARG)malloc(sizeof(MB_POSTARG));
	pPostArg->bstrKey = NULL;
	pPostArg->bstrValue = NULL;
	pPostArg->bBinary = FALSE;

	*ppPostArg = pPostArg;
	return TRUE;
};
inline BOOL DeletePostArg(PMB_POSTARG pPostArg)
{
	if (pPostArg==NULL) return FALSE;
	if (pPostArg->bstrKey)
	{
		SysFreeString(pPostArg->bstrKey);
		pPostArg->bstrKey = NULL;
	}
	if (pPostArg->bstrValue)
	{
		SysFreeString(pPostArg->bstrValue);
		pPostArg->bstrValue = NULL;
	}
	free(pPostArg);
	return TRUE;
};
typedef CSimpleArray<PMB_POSTARG>	MB_POSTARG_ARR, *PMB_POSTARG_ARR;
class CNPPostArg
{
public:
	CNPPostArg() {}
	~CNPPostArg() {RemoveAll();}

	void	AddArgument(LPCTSTR strKey, LPCTSTR strValue);
	void	AddFile(LPCTSTR strKey, LPCTSTR strFilePath);

	MB_POSTARG_ARR	m_arrPostArg;

protected:	
	void	RemoveAll();
};
/**
* @group 	PCM Base Lib	
* @brief 	Generic HTTP Client
* @date		08/22/2006
* @author	telltale
* @remark	
*/
class CNPHTTPClient
{
public:
	/**
	* @brief 	get lastmodifieddate & contentlength
	* @param	pszcUrl		target url
	* @param	pSysTimeLastUpdate	return lastmodifieddate
	* @param	plSize				return contentlength	
	* @return	success
	*/
	BOOL	GetLastUpdate(LPCTSTR pszcUrl, SYSTEMTIME* pSysTimeLastUpdate, long* plSize);
	/**
	* @brief 	GET command
	* @param	pszcUrl		target url
	* @param	pBuf		buffer
	* @param	dwMaxBuf	buffer max length
	* @param	pszCookie	cookie
	* @return	success
	*/
	BOOL	GetPage(LPCTSTR pszcUrl, BYTE* pBuf, DWORD dwMaxBuf, LPCTSTR pszcCookie=NULL);
	
	/**
	* @brief 	POST command and download file
	* @param	pszcUrl		target url
	* @param	pszcDownloadPath		download file path
	* @param	dwParam		user's data
	* @param	pfn			callback see PFNHTTPSTATUSCALLBACK
	* @return	
	*/
	/*! \brief : POST command and download file */
	BOOL	DownloadFile(LPCTSTR pszcUrl, LPCTSTR pszcDownloadPath, DWORD_PTR dwParam=0, PFNHTTPSTATUSCALLBACK pfn=NULL);

	///< upload file by multipart
	BOOL	UploadFile(const TCHAR *strUrl, PMB_POSTARG_ARR pPostArg, LPCTSTR pszCookie, TCHAR* strPage, int nMaxPage, DWORD_PTR dwParam=0, PFNHTTPSTATUSCALLBACK lpCallback=NULL);

	///< POST
	BOOL	PostPage(const TCHAR *strUrl, PMB_POSTARG_ARR pPostArg, LPCTSTR pszCookie, TCHAR* strPage, int nMaxPage, DWORD_PTR dwParam=0, PFNHTTPSTATUSCALLBACK lpCallback=NULL);

public:
	CNPHTTPClient(void);
	~CNPHTTPClient(void);

private:
	BOOL		ReceiveData(HINTERNET hIS, BYTE* pBuf, DWORD dwMaxBuf);
	BOOL		DownloadFile(HINTERNET hIS, LPCTSTR pszcDownloadPath, DWORD_PTR dwParam=0, PFNHTTPSTATUSCALLBACK pfn=NULL);	
	DWORD		SendMultiPartsFormData(HINTERNET hRO, PMB_POSTARG_ARR pPostArgArr, DWORD_PTR dwParam=0, PFNHTTPSTATUSCALLBACK lpCallback=NULL);
	DWORD		GetMultiPartsFormDataLength(PMB_POSTARG_ARR pPostArgArr);
	DWORD		GetUrlEncodedFromArg(LPTSTR pszContent, PMB_POSTARG_ARR pPostArgArr);
	LPCTSTR		GetContentType(LPCTSTR szName);
	BOOL		ReceiveData(HINTERNET hIS, TCHAR* strPage, int nMaxPage);
};
